_TYPE_MAP = {
	'CharField': 'string',
	'TextField': 'string',
	'SmallIntegerField': 'int',
	'PositiveSmallIntegerField': 'int',
	'IntegerField': 'int',
	'PositiveIntegerField': 'int',
	'BigIntegerField': 'int',
	'DecimalField': 'float',
	'FloatField': 'float',
	'BooleanField': 'bool',
	'URLField': 'string',
	'EmailField': 'string',
	'DateField': 'date',
	'TimeField': 'date',
	'DateTimeField': 'date',
	'JSONField': 'object',
	'ArrayField': 'array',
	'IPAddressField': 'string',
	'GenericIPAddressField': 'string'
}

_FORMAT_MAP = {
	'URLField': 'url',
	'EmailField': 'email',
	'DateField': 'date',
	'TimeField': 'time',
	'DateTimeField': 'datetime',
	'IPAddressField': 'ip',
	'GenericIPAddressField': 'ip'
}

_MIN_MAP = {
	'SmallIntegerField': -32768,
	'PositiveSmallIntegerField': 0,
	'IntegerField': -2147483648,
	'PositiveIntegerField': 0,
	'BigIntegerField': -9223372036854775808,
}

_MAX_MAP = {
	'SmallIntegerField': 32767,
	'PositiveSmallIntegerField': 32767,
	'IntegerField': 2147483647,
	'PositiveIntegerField': 2147483647,
	'BigIntegerField': 9223372036854775807,
}

class ApiFieldRule(object):
	def __init__(self, field):
		# Only one rule set per field
		rule = {}
		type_name = field.__class__.__name__
		rule['type'] = _TYPE_MAP.get(type_name, 'string')
		if not field.choices:
			minimum = self._get_field_min(field)
			if minimum is not None:
				rule['min'] = minimum
			maximum = self._get_field_max(field)
			if maximum is not None:
				rule['max'] = maximum
			regex = self._get_regex(field)
			format = _FORMAT_MAP.get(type_name)
			if regex and format:
				rule['format'] = [format, regex]
			else:
				rule['format'] = format or regex

		if not field.blank:
			rule['required'] = True
		self.rule = rule

	def _get_regex(self, field):
		for validator in field.validators:
			if validator.__class__.__name__ == 'RegexValidator':
				return validator.regex.pattern

	def _get_field_min(self, field):
		for validator in field.validators:
			if validator.__class__.__name__ in ('MinValueValidator', 'MinLengthValidator'):
				return validator.limit_value
		type_name = field.__class__.__name__
		return _MIN_MAP.get(type_name)

	def _get_field_max(self, field):
		for validator in field.validators:
			if validator.__class__.__name__ in ('MaxValueValidator', 'MaxLengthValidator'):
				return validator.limit_value
		type_name = field.__class__.__name__
		return _MAX_MAP.get(type_name) or getattr(field, 'max_length', None)



import random
from optparse import make_option

from django.conf import settings
from django.core.management.base import BaseCommand

class Command(BaseCommand):
	help = 'Obfuscate specified settings, keys, secrets etc. for use in mobile apps.'
	option_list = BaseCommand.option_list + (
			make_option('--ios',
				action='store_true',
				dest='ios',
				default=False,
				help='Output obfuscated iOS code.'),
			make_option('--android',
				action='store_true',
				dest='android',
				default=False,
				help='Output obfuscated Android code.')
		)

	def handle(self, *args, **options):
		random.seed(None)
		for arg in args:
			if hasattr(settings, arg):
				setting = getattr(settings, arg)
				x = random.randint(0, 255)
				bytes = map(lambda b: hex(ord(b) ^ x), [ch for ch in setting])
				if options['ios']:
					print "%s: [API deobfuscate:(unsigned char []){%s,0} size:%d x:%s]" % (arg, ','.join(bytes), len(bytes) + 1, hex(x))
				elif options['android']:
					print "%s: API.deobfuscate(new byte[] {(byte)%s}, %s)" % (arg, ',(byte)'.join(bytes), hex(x))



import StringIO

from django.utils.html import escape

def __dump_value(t, value, tag, file):
	if t == str or t == unicode:
		file.write('<%s>%s</%s>' % (tag, escape(value), tag))
	elif t == bool:
		file.write('<%s>%s</%s>' % (tag, str(value).lower(), tag))
	else:
		file.write('<%s>%s</%s>' % (tag, str(value), tag))

def __dump_dict(dictionary, file):
	"""Output a dict."""
	for key, value in dictionary.items():
		t = type(value)
		if t == dict:
			file.write('<div class="%s">' % key)
			__dump_dict(value, file)
			file.write('</div>')
		elif t == tuple or t == list or t == set:
			file.write('<ul class="%s">' % key)
			__dump_array(value, file)
			file.write('</ul>' % key)
		else:
			__dump_value(t, value, key, file)

def __dump_array(array, file):
	"""Output an array."""
	for value in array:
		t = type(value)
		if t == dict:
			__dump_dict(value, file)
		elif t == tuple or t == list or t == set:
			file.write('<ul>')
			__dump_array(value, file)
			file.write('</ul>')
		else:
			__dump_value(t, value, 'li', file)

def dumps(data, file=None):
	"""Similar to json.dumps, will return an html fragment string."""
	if file:
		output = file
	else:
		output = StringIO.StringIO()
	__dump_array([data], output)
	if not file:
		return output.getvalue()



from django.apps import apps
from django.conf import settings
from django.db.models import Q, QuerySet

from .functions import sanitize_order_by, _get_api_model
from .response import set_response_headers

get_model = apps.get_model

"""
In Django 1.9 _clone method no longer allows a different class. Instead need to subclass the ModelIterable class
Differences described below when the queryset is iterated:
1.8:
__iter__ -> _fetch_all -> list(self.iterator()) -> key generator code (QuerySet.iterator) -> the remainder code provided below to cast objects (SubclassQuerySet.iterator)
1.9:
__iter__ -> _fetch_all -> list(self.iterator()) -> return iter(self._iterable_class(self)) -> key generator code (ModelIterable.__iter__()) is almost identical to the key generator code from 1.8 and can be wrapped the same

Since the Iterable class is not created until needed it is possible to simly replace it.
See for example the values() method:
clone._iterable_class = ValuesIterable
"""

class SubclassQuerySet(QuerySet):
	"""QuerySet that returns subclasses automatically when iterating."""
	def _clone(self, **kwargs):
		clone = super(SubclassQuerySet, self)._clone(**kwargs)
		clone._subclass_attrs = getattr(self, '_subclass_attrs', None)
		return clone
	def iterator(self):
		iter = super(SubclassQuerySet, self).iterator()
		for obj in iter:
			sub_obj = None
			for attr in self._subclass_attrs:
				if hasattr(obj, attr):
					sub_obj = getattr(obj, attr)
					break
			if sub_obj:
				yield sub_obj
			else:
				yield obj

def subclass_filter(*subclasses, **names):
	"""
	Convert the QuerySet to a SubclassQuerySet, where only the specified subclasses are returned and each object is cast to its subclass.
	The keyword arguments, has the names for the collection when generating files. name and name_plural provided as CamelCase are required.
	"""
	# TODO: To support multi-level inheritance, subclass_attrs needs find the common parent of all subclasses and will either be a dict tree of keys only (multi-level) or list (single).
	# The iterator() will then do multi-level getattr if isinstance(self._subclass_attrs, dict) otherwise use the existing code.
	# The select_related() part will also need to specify multi-level inheritance using intermediate__subclass strings
	subclasses = list(subclasses)
	for i, subclass in enumerate(subclasses):
		if isinstance(subclass, (str, unicode)):
			subclass = subclass.split('.')
			subclasses[i] = get_model(subclass[0], subclass[1])
	subclass_attrs = [cls.__name__.lower() for cls in subclasses]
	def subclass_filter_inner(request, queryset):
		set_response_headers(request, **{'X-Mixed-Results': True})
		queryset = queryset._clone(SubclassQuerySet).select_related(*subclass_attrs)
		# Remember the subclass arguments, because the queryset.query.select_related dict may have other non-subclass attributes
		queryset._subclass_attrs = subclass_attrs
		return queryset
	return subclass_filter_inner

def search_filter(request, queryset):
	"""Filter down the result by using a q query parameter. API.search_fields MUST be set to use this."""
	query = request.GET.get('q')
	if query and hasattr(queryset.model, 'API') and hasattr(queryset.model.API, 'search_fields') and queryset.model.API.search_fields:
		params = None
		for field in queryset.model.API.search_fields:
			if field.find('.'):
				field = '__'.join(field.split('.'))
			if not params:
				params = Q(**{field + '__icontains': query})
			else:
				params = params | Q(**{field + '__icontains': query})
		queryset = queryset.filter(params)
	return queryset

def field_filter(request, queryset):
	"""Filter the results by named fields in the query string. API.filter_fields MUST be set to use this."""
	if len(request.GET) and hasattr(queryset.model, 'API') and hasattr(queryset.model.API, 'filter_fields'):
		model = _get_api_model(queryset.model)
		for field in queryset.model.API.filter_fields:
			value = request.GET.get(field)
			if value:
				encoded_field = model.encoded_fields[field]
				key = encoded_field[0]
				decode = encoded_field[3]
				if decode:
					value = decode(value)
				queryset = queryset.filter(**{key: value})
	return queryset

def order_by_filter(request, queryset):
	"""Order the results by using an orderby parameter."""
	order_by = sanitize_order_by(request.GET.get('orderby', ''))
	if order_by:
		if hasattr(queryset.model, 'API') and hasattr(queryset.model.API, 'order_by_fields') and queryset.model.API.order_by_fields:
			if order_by.lstrip('-') in queryset.model.API.order_by_fields:
				queryset = queryset.order_by(order_by)
		else:
			queryset = queryset.order_by(order_by)
	return queryset

__API_PAGE_SIZE = getattr(settings, 'API_PAGE_SIZE', 100)

def paginate_filter(request, queryset):
	"""Paginate the results based on page and pagesize parameters. This should be the last filter applied."""
	page = request.GET.get('page', 0)
	if not isinstance(page, int):
		if page.isdigit():
			page = int(page)
		else:
			return queryset.none()
	if hasattr(queryset.model, 'API') and hasattr(queryset.model.API, 'page_size'):
		max_page_size = queryset.model.API.page_size
	else:
		max_page_size = __API_PAGE_SIZE
	page_size = request.GET.get('pagesize', max_page_size)
	if not isinstance(page_size, int):
		if page_size.isdigit():
			page_size = int(page_size)
		else:
			return queryset.none()
	if page_size > max_page_size or page_size < 0:
		page_size = max_page_size
	start_index = page * page_size
	end_index = start_index + page_size
	total = queryset.count()
	total_pages = total/page_size + (1 if total % page_size else 0)
	set_response_headers(request, **{'X-Total': total, 'X-Total-Pages': total_pages, 'X-Page': page, 'X-Page-Size': page_size})
	if (page + 1) < total_pages:
		set_response_headers(request, **{'X-Next-Page': page + 1})
	if page:
		set_response_headers(request, **{'X-Prev-Page': page - 1})
	return queryset[start_index:end_index]

def combine_filters(*filters):
	"""Combine multiple filters into one."""
	def combine_filters_inner(request, queryset):
		for filter in filters:
			queryset = filter(request, queryset)
		return queryset
	return combine_filters_inner

def filter_as_authorization(model, filter):
	"""Create an authorization function based on a collection filter."""
	if isinstance(model, (str, unicode)):
		model = model.split('.')
		model = get_model(model[0], model[1])
	def filter_as_authorization_fun(request, obj):
		return filter(request, model.objects.all()).filter(id=obj.id).exists()
	filter_as_authorization_fun.__doc__ = "Only allow access to objects in the filter: %s" % (filter.__doc__ if filter.__doc__ else filter.__name__ + '()')
	return filter_as_authorization_fun



import os
from optparse import make_option

from django.core.management.base import BaseCommand
from django.template import Template

from symmetric.management.generatemodels import GenerateModelsCommand
from symmetric.management.translate import data_to_objc

included_field_setter = """{% if included %}- (void)set{{ name|title }}:({{ included_name }} *)new{{ name|title }}
{
	if([new{{ name|title }} isKindOfClass:[NSDictionary class]])
	{
		NSDictionary *dictionary = (NSDictionary *)new{{ name|title }};
		new{{ name|title }} = [[[{{ prefix }}{{ included_name }} alloc] init] autorelease];
		[new{{ name|title }} setValuesForKeysWithDictionary:dictionary];
	}
	if(_{{ name }} != new{{ name|title }})
	{
		[_{{ name }} release];
		_{{ name }} = [new{{ name|title }} retain];
	}
}
{% endif %}"""

class Command(BaseCommand, GenerateModelsCommand):
	help = 'Generate iOS models for API models or endpoints.'
	option_list = BaseCommand.option_list + GenerateModelsCommand.option_list

	def handle(self, *args, **options):
		# Create the templates
		self.templates = []
		template_path = os.path.join(os.path.normpath(os.path.dirname(__file__) + '/../templates'), 'ios-model.h')
		with open(template_path) as f:
			self.templates.append(Template(f.read()))
		template_path = os.path.join(os.path.normpath(os.path.dirname(__file__) + '/../templates'), 'ios-model.m')
		with open(template_path) as f:
			self.templates.append(Template(f.read()))
		self.template_extensions = ['h', 'm']

		# Create the mappings
		import_mapping = {
			'ForeignKey': Template('{% if included %}#import "{{prefix}}{{included_name}}.h"{% endif %}'),
			'Field': ''
		}
		property_base = "@property (nonatomic, %s{%% if name == 'hash' or name == 'description' %%}, getter=get{{name|title}}{%% endif %%}) %s{{name}};"
		property_mapping = {
			'CharField': Template(property_base % ('copy', 'NSString *')),
			'IntegerField': Template(property_base % ('assign', 'int32_t ')),
			'PositiveIntegerField': Template(property_base % ('assign', 'uint32_t ')),
			'BigIntegerField': Template(property_base % ('assign', 'int64_t ')),
			'DateField': Template(property_base % ('retain', 'NSDate *')),
			'DecimalField': Template(property_base % ('assign', 'float ')),
			'FloatField': Template(property_base % ('assign', 'float ')),
			'BooleanField': Template(property_base % ('assign', 'BOOL ')),
			'AutoField': Template(property_base % ('assign', 'uint32_t ')),
			'ForeignKey': Template('@property (nonatomic, {% if included %}retain{% else %}assign{% endif %}) {% if included %}{{prefix}}{{included_name}} *{% else %}uint32_t {% endif %}{{name}};'),
			'JSONField': Template(property_base % ('retain', 'NSMutableDictionary *')),
			'ArrayField': Template(property_base % ('retain', 'NSMutableArray *')),
			'Field': '#error "Unsupported field for {name}"'
		}
		synthesizer_mapping = {
			'Field': Template("{% if name == 'hash' or name == 'description' %}@synthesize {{name}} = _{{name}};{% endif %}")
		}
		default_values_data = lambda format_context: '_%s = [%s retain];' % (format_context['name'], data_to_objc(format_context['default'])) if format_context.get('default') else ''
		default_values_mapping = {
			'CharField': default_values_data,
			'JSONField': default_values_data,
			'ArrayField': default_values_data,
			'BooleanField': Template('{% if default %}_{{name}} = YES;{% endif %}'),
			'Field': Template('{% if default %}_{{name}} = {{ default }};{% endif %}')
		}
		dealloc_format = '[_{name} release];'
		dealloc_mapping = {
			'CharField': dealloc_format,
			'DateField': dealloc_format,
			'ForeignKey': Template('{% if included %}[_{{name}} release];{% endif %}'),
			'JSONField': dealloc_format,
			'ArrayField': dealloc_format,
			'Field': ''
		}
		encode_base = '[encoder encode%s:_{{name}} forKey:@"{{name}}"];'
		encode_mapping = {
			'AutoField': Template(encode_base % 'Int32'),
			'CharField': Template(encode_base % 'Object'),
			'IntegerField': Template(encode_base % 'Int32'),
			'BigIntegerField': Template(encode_base % 'Int64'),
			'DateField': Template(encode_base % 'Object'),
			'DecimalField': Template(encode_base % 'Float'),
			'FloatField': Template(encode_base % 'Float'),
			'BooleanField': Template(encode_base % 'Bool'),
			'ForeignKey': Template(encode_base % '{% if included %}Object{% else %}Int32{% endif %}'),
			'JSONField': Template(encode_base % 'Object'),
			'ArrayField': Template(encode_base % 'Object'),
			'Field': ''
		}
		decode_base = '%s{{name}} = [decoder decode%sForKey:@"{{name}}"];'
		decode_mutable_base = '%s{{name}} = [[[decoder decode%sForKey:@"{{name}}"] mutableCopy] autorelease];'
		decode_mapping = {
			'AutoField': Template(decode_base % ('_', 'Int32')),
			'CharField': Template(decode_base % ('self.', 'Object')),
			'IntegerField': Template(decode_base % ('_', 'Int32')),
			'BigIntegerField': Template(decode_base % ('_', 'Int64')),
			'DateField': Template(decode_base % ('self.', 'Object')),
			'DecimalField': Template(decode_base % ('_', 'Float')),
			'FloatField': Template(decode_base % ('_', 'Float')),
			'BooleanField': Template(decode_base % ('_', 'Bool')),
			'ForeignKey': Template(decode_base % ('{% if included %}self.{% else %}_{% endif %}', '{% if included %}Object{% else %}Int32{% endif %}')),
			'JSONField': Template(decode_mutable_base % ('self.', 'Object')),
			'ArrayField': Template(decode_mutable_base % ('self.', 'Object')),
			'Field': ''
		}
		url_encoded_data_string = 'if(_{{name}})\n\t[args addObject:[NSString stringWithFormat:@"{{name}}=%@", [_{{name}} stringByAddingPercentEncodingWithAllowedCharacters:NSCharacterSet.URLQueryAllowedCharacterSet]]];{% if null %}\nelse\n\t[args addObject:@"{{name}}=NULL"];{% endif %}'
		url_encoded_data_date = 'if(_{{name}})\n\t[args addObject:[NSString stringWithFormat:@"{{name}}=%@", [[[API dateFormatter] stringFromDate:_{{name}}] stringByAddingPercentEncodingWithAllowedCharacters:NSCharacterSet.URLQueryAllowedCharacterSet]]];{% if null %}\nelse\n\t[args addObject:@"{{name}}=NULL"];{% endif %}'
		url_encoded_data_number = '[args addObject:[NSString stringWithFormat:@"{{name}}=%s", _{{name}}]];'
		url_encoded_data_json = 'if(_{{name}})\n\t[args addObject:[NSString stringWithFormat:@"{{name}}=%@", [[NSString stringWithJSONObject:_{{name}}] stringByAddingPercentEncodingWithAllowedCharacters:NSCharacterSet.URLQueryAllowedCharacterSet]]];{% if null %}\nelse\n\t[args addObject:@"{{name}}=NULL"];{% endif %}'
		url_encoded_data_json_included =  '[args addObject:@"{{name}}=%@", [[NSString stringWithJSONObject:_{{name}}] stringByAddingPercentEncodingWithAllowedCharacters:NSCharacterSet.URLQueryAllowedCharacterSet]];'
		url_encoded_data_null = '[args addObject:@"{{name}}=NULL"];'
		url_encoded_data_mapping = {
			'CharField': Template(url_encoded_data_string),
			'IntegerField': Template(url_encoded_data_number % '%d'),
			'PositiveIntegerField': Template(url_encoded_data_number % '%u'),
			'BigIntegerField': Template(url_encoded_data_number % '%lld'),
			'DateField': Template(url_encoded_data_date),
			'DecimalField': Template(url_encoded_data_number % '%f'),
			'FloatField': Template(url_encoded_data_number % '%f'),
			'BooleanField': '[args addObject:[NSString stringWithFormat:@"{name}=%@", (_{name}?@"true":@"false")]];',
			'ForeignKey': Template('{% if included %}{% if included_readonly %}if(_{{included_obj_name}}) [args addObject:[NSString stringWithFormat:@"{{name}}=%u", _{{included_obj_name}}.objectId]]; else ' + url_encoded_data_null + ' {% else %}' + url_encoded_data_json_included + '{% endif %}{% else %}if(!_{{name}}) ' + url_encoded_data_null + ' else ' + (url_encoded_data_number % '%u') + '{% endif %}'),
			'JSONField': Template(url_encoded_data_json),
			'ArrayField': Template(url_encoded_data_json),
			'Field': '',
			'WriteOnly': True
		}
		dict_field_mapping = {
			'ForeignKey': Template('{% if not included %}{name}{% endif %}'),
			'Field': '{name}',
			'WriteOnly': True
		}
		dict_included_mapping = {
			'ForeignKey': Template('{% if included %}{% if included_readonly %}[dictionary setObject:((_{{included_obj_name}}) ? [NSNumber numberWithUnsignedInteger:_{{included_obj_name}}.objectId] : [NSNull null]) forKey:@"{{name}}"];{% else %}[dictionary setObject:(_{{name}} ? [_{{name}} dictionary] : [NSNull null]) forKey:@"{{name}}"];{% endif %}{% endif %}'),
			'Field': '',
			'WriteOnly': True
		}
		included_field_setter_mapping = {
			'ForeignKey': Template(included_field_setter),
			'Field': ''
		}

		self.lang = 'objc'
		self.property_types = {int: 'NSInteger ', float: 'float ', str: 'NSString *', bool: 'BOOL '}
		self.property_declarations = "@property (nonatomic, readonly) {type}{name};"
		self.property_implementations = Template('- ({{type.strip}}){{name}}\n{\n\treturn {{code}};\n}\n')
		self.mappings = {'imports': import_mapping, 'properties': property_mapping, 'synthesizers': synthesizer_mapping, 'default_values': default_values_mapping, 'dealloc': dealloc_mapping, 'encode': encode_mapping, 'decode': decode_mapping, 'url_encoded_data': url_encoded_data_mapping, 'dict_fields': dict_field_mapping, 'dict_included': dict_included_mapping, 'included_field_setters': included_field_setter_mapping }
		self.expand_mappings('CharField', 'TextField', 'IPAddressField', 'GenericIPAddressField')
		self.render(*args, **options)




def api_property(code, return_type=int, translations=None):
	def api_property_fun(self):
		return eval(code)
	api_property_fun.api_code = code
	api_property_fun.api_type = return_type
	api_property_fun.api_translations = translations
	return property(api_property_fun)

def create_model(name, fields):
	from django.db import models
	"""Create a model with a dict of named fields."""
	fields = dict(fields)
	fields['__module__'] = 'api.models'
	return type(name, (models.Model,), fields)

def clone_model(model, name):
	"""Clone a model with same fields and subclasses, just changing the name. Requires 1.7+ for field.clone()"""
	fields = {'__module__': model.__module__}
	for field in model._meta.fields:
		if field.model is model and not field.auto_created:
			fields[field.name] = field.clone()
	# Clone the api properties
	for cls in [model] + list(model._meta.get_parent_list()):
		for field_name in cls.__dict__:
			attr = cls.__dict__[field_name]
			if type(attr) is property and attr.fget and hasattr(attr.fget, 'api_code'):
				fields[field_name] = api_property(attr.fget.api_code, attr.fget.api_type, attr.fget.api_translations)
	return type(name, model.__bases__, fields)

def get_related_model(field):
	try:
		return field.related.parent_model
	except AttributeError:
		# Django 1.8+
		return field.related.model



import re
from importlib import import_module
from optparse import make_option

from django.conf import settings
from django.core.management.base import BaseCommand

import symmetric.management.overrides
from symmetric.filters import combine_filters
from symmetric.functions import _ApiModel, get_object_data
from symmetric.management.functions import get_doc_str, get_resource_type, format_regex_stack, is_anonymous, is_auto_now
from symmetric.views import api_view, api_related_view, api_filter_contacts_view
from symmetric.views import api_login_view, api_logout_view, api_create_user_view, api_set_password_view, api_reset_password_view
from symmetric.views import ApiAction, ApiRequirement, BasicApiView

class Command(BaseCommand):
	help = 'Generate documentation for API endpoints.'
	option_list = BaseCommand.option_list + (
			make_option('--compact', '-c',
				action='store_true',
				dest='compact',
				default=False,
				help='Output only callback information info with the url endpoints.'),
			make_option('--urls', '-u',
				action='store_true',
				dest='urls',
				default=False,
				help='Output only the url endpoints.'),
			make_option('--anonymous', '-a',
				action='store_true',
				dest='anonymous',
				default=False,
				help='Output only the endpoints that may be used anonymously without a login session.'),
			make_option('--raml', '-r',
				action='store_true',
				dest='raml',
				default=False,
				help=''),
			make_option('--raml-mixin', '-m',
				type='string',
				dest='raml-mixin',
				help='Mixin/overwrite the raml with following raml file.')
		)
	API_AUTH_VIEWS = (api_login_view, api_logout_view, api_create_user_view, api_set_password_view, api_reset_password_view)

	def format_actions(self, view, resource_type):
		# Mask the actual allowed actions based on the resource type
		masks = {'Single Object': ApiAction.READ | ApiAction.UPDATE, 'Collection': ApiAction.READ | ApiAction.CREATE, 'Object': ApiAction.READ | ApiAction.UPDATE | ApiAction.DELETE}
		actions = view.actions & masks[resource_type]

		actions_str = []
		if actions & ApiAction.READ:
			actions_str.append('READ')
		if actions & ApiAction.UPDATE:
			actions_str.append('UPDATE')
		if actions & ApiAction.CREATE:
			actions_str.append('CREATE')
		if actions & ApiAction.DELETE:
			actions_str.append('DELETE')
		return ', '.join(actions_str)

	def format_requirements(self, view):
		requirements = []
		if view.requirements & ApiRequirement.LOGIN:
			requirements.append('User must be logged in.')
		if view.requirements & ApiRequirement.STAFF:
			requirements.append('User must be staff.')
		if view.requirements & ApiRequirement.SUPERUSER:
			requirements.append('User must be superuser.')
		if view.requirements & ApiRequirement.JSONP:
			requirements.append('JSONP requests allowed.')
		if view.requirements & ApiRequirement.HMAC:
			requirements.append('CREATE and UPDATE requests must include an HMAC.')
		if view.requirements & ApiRequirement.ANONYMOUS_READ:
			requirements.append('READ requests may be anonymous.')
		if view.requirements & ApiRequirement.HTTPS:
			requirements.append('Request must be done over HTTPS.')
		return ' '.join(requirements)

	def help_text(self, field_name, view):
		for field in view.model._meta.fields:
			if field.name == field_name:
				if hasattr(field, 'help_text'):
					return field.help_text.encode('utf-8')
		return ''

	def build_raml(self, view):
		# TODO: support BasicApiView
		if isinstance(view, BasicApiView):
			return
		resource = None
		path = ['/' + c.replace('<','{').replace('>','}') for c in format_regex_stack(self.regex_stack).split('/') if c]
		for component in path[1:]:
			if resource is None:
				resource = self.raml_data
			if not resource.has_key(component):
				resource[component] = {}
			resource = resource[component]
		resource['displayName'] = view.model.__name__
		if view.actions & ApiAction.READ:
			resource['get'] = {'responses': {200: {'body': {'*/*':{}}}}}

	def format_model(self, view):
		if hasattr(view, 'parent_model') and view.parent_model:
			# api_related_view and BasicApiView
			print 'Parent Model: %s' % view.parent_model.__name__
		print 'Model: %s' % view.model.__name__
		if view.model.__dict__.has_key('clean'):
				print 'Clean: %s' % get_doc_str(view.model.clean)
		if view.model.__dict__.has_key('save'):
			print 'Save: %s' % get_doc_str(view.model.save)
		if self.compact:
			return
		model = _ApiModel(view.model)
		print 'Fields: '
		print '%s%s%s%s' % ('name'.center(25), 'list'.center(10), 'readonly'.center(15), 'sub-object'.center(10))
		for name, encoded_name, encode, decode in model.fields:
			list_str = ''
			for name_list, encoded_name_list, encode_list, decode_list in model.list_fields:
				if name == name_list:
					list_str = '*'
					break
			sub_object_str = '*' if encode is get_object_data else ''
			if not model.encoded_fields.has_key(encoded_name):
				if is_auto_now(name, view):
					readonly_str = 'auto'
				else:
					readonly_str = '*'
			else:
				# Find out if it is automatically set from request_user_field or request_ip_field or is a related_field
				if hasattr(view.model, 'API') and hasattr(view.model.API, 'request_user_field') and view.model.API.request_user_field == name:
					readonly_str = 'auto'
				elif hasattr(view.model, 'API') and hasattr(view.model.API, 'request_ip_field') and view.model.API.request_ip_field == name:
					readonly_str = 'auto'
				elif isinstance(view, api_related_view) and view.related_field == name[:-3]:
					readonly_str = 'auto'
				else:
					readonly_str = ''
			help = self.help_text(name, view)
			print '%s%s%s%s   %s' % (encoded_name.ljust(25), list_str.center(10), readonly_str.center(15), sub_object_str.center(10), help)

	def format_view(self, view, resource_type):
		path = format_regex_stack(self.regex_stack)
		if self.docfilter and path.lower().find(self.docfilter) == -1 and (not view.model or view.model.__name__.lower() != self.docfilter):
			return
		if self.urls:
			print path
			return
		print 'Path: %s' % path
		if isinstance(view, BasicApiView) and view.__call__.__doc__:
			print view.__call__.__doc__
		print 'Allowed Actions: [%s]' % self.format_actions(view, resource_type)
		print 'Resource Type: ' + resource_type
		if view.requirements:
			print 'Requirements: %s' % self.format_requirements(view)
		if hasattr(view, 'filter') and view.filter:
			if isinstance(view.filter, combine_filters):
				print 'Filters:'
				print str(view.filter)
			else:
				print 'Filter: %s' % get_doc_str(view.filter)
		if hasattr(view, 'authorization') and view.authorization:
			print 'Authorization: %s' % get_doc_str(view.authorization)
		if hasattr(view, 'verification') and view.verification:
			print 'Verification: %s' % get_doc_str(view.verification)
		if view.model:
			self.format_model(view)
		print '-----------------------------------------------------------------------'

	def format_auth_view(self, view):
		path = format_regex_stack(self.regex_stack)
		if self.docfilter:
			return
		if self.urls:
			print path
			return
		print 'Path: %s' % path
		print get_doc_str(view)
		if hasattr(view, 'requirements') and view.requirements:
			print 'Requirements: %s' % self.format_requirements(view)
		if hasattr(view, 'fields') and view.fields:
			print 'Filter Fields: %s' % ','.join(view.fields)
		if hasattr(view, 'model') and view.model:
			self.format_model(view)
		print '-----------------------------------------------------------------------'

	def enum_patterns(self, patterns):
		for pattern in patterns:
			if pattern.callback:
				if self.anonymous and not is_anonymous(pattern.callback):
					continue
				if isinstance(pattern.callback, (api_view, BasicApiView)):
					self.regex_stack.append(pattern._regex)
					if self.raml:
						self.build_raml(pattern.callback)
					else:
						self.format_view(pattern.callback, get_resource_type(self.regex_stack, pattern))
					self.regex_stack.pop()
				elif pattern.callback in Command.API_AUTH_VIEWS or isinstance(pattern.callback, api_filter_contacts_view):
					self.regex_stack.append(pattern._regex)
					self.format_auth_view(pattern.callback)
					self.regex_stack.pop()
			else:
				self.regex_stack.append(pattern._regex)
				self.enum_patterns(pattern.url_patterns)
		if self.regex_stack:
			self.regex_stack.pop()

	def handle(self, *args, **options):
		if args:
			self.docfilter = args[0].lower()
		else:
			self.docfilter = None
		self.urls = options['urls']
		self.compact = options['compact']
		self.anonymous = options['anonymous']
		self.raml = options['raml']
		self.regex_stack = []
		self.raml_data = {}
		module = import_module(settings.ROOT_URLCONF)
		self.enum_patterns(module.urlpatterns)
		if self.raml:
			try:
				import yaml
				self.raml_data.update({'title': 'API', 'version': 'v1', 'baseUri':'/api/{version}'})
				if options['raml-mixin']:
					with open(options['raml-mixin']) as f:
						self.raml_data.update(yaml.load(f))
				print '#%RAML 0.8'
				print yaml.dump(self.raml_data, default_flow_style=False, allow_unicode=True, encoding=None)
			except ImportError as e:
				print "Please install pyyaml first."



import StringIO

from django.db.models import Model
from django.db.models.query import QuerySet
from django.utils.html import escape

def __get_array_tag(array, tag='values'):
	if isinstance(array, QuerySet):
		return array.model._meta.verbose_name_plural.replace(' ', '').lower()
	return tag

def __dump_value(t, value, tag, file, default=None):
	if issubclass(t, (str, unicode)):
		file.write('<%s>%s</%s>' % (tag, escape(value), tag))
	elif issubclass(t, bool):
		file.write('<%s>%s</%s>' % (tag, str(value).lower(), tag))
	elif issubclass(t, (int, long, float)):
		file.write('<%s>%s</%s>' % (tag, str(value), tag))
	elif value is None:
		file.write('<%s></%s>' % (tag, tag))
	else:
		# Unknown value
		if default:
			new_value = default(value)
			if new_value:
				t = type(new_value)
				if issubclass(t, dict):
					tag = value.__class__.__name__.lower()
					file.write('<%s>' % tag)
					__dump_dict(new_value, file, default)
					file.write('</%s>' % tag)
				elif issubclass(t, (tuple, list, set, QuerySet)):
					tag = __get_array_tag(new_value, tag)
					file.write('<%s>' % tag)
					__dump_array(new_value, file, default)
					file.write('</%s>' % tag)
				else:
					__dump_value(t, new_value, tag, file, default)
		else:
			raise TypeError(repr(obj) + " is not serializable")

def __dump_dict(dictionary, file, default=None):
	"""Output a dict."""
	for key, value in dictionary.items():
		t = type(value)
		if issubclass(t, dict):
			file.write('<%s>' % key)
			__dump_dict(value, file, default)
			file.write('</%s>' % key)
		elif issubclass(t, (tuple, list, set, QuerySet)):
			file.write('<%s>' % key)
			__dump_array(value, file, default)
			file.write('</%s>' % key)
		else:
			__dump_value(t, value, key, file, default)

def __dump_array(array, file, default=None):
	"""Output an array."""
	for value in array:
		t = type(value)
		if issubclass(t, dict):
			file.write('<values>')
			__dump_dict(value, file, default)
			file.write('</values>')
		elif issubclass(t, (tuple, list, set, QuerySet)):
			tag = __get_array_tag(value)
			file.write('<%s>' % tag)
			__dump_array(value, file, default)
			file.write('</%s>' % tag)
		else:
			__dump_value(t, value, 'value', file, default)

def dumps(data, file=None, default=None):
	"""Similar to json.dumps, will return an xml string."""
	if file:
		output = file
	else:
		output = StringIO.StringIO()
	output.write('<?xml version="1.0" encoding="UTF-8" ?>')
	t = type(data)
	if issubclass(t, dict):
		output.write('<data>')
		__dump_dict(data, output, default)
		output.write('</data>')
	elif issubclass(t, (tuple, list, set, QuerySet)):
		output.write('<data>')
		__dump_array(data, output, default)
		output.write('</data>')
	else:
		__dump_value(t, data, 'data', output, default)
	if not file:
		return output.getvalue()



import json

from django.conf import settings
from django.http import HttpRequest, QueryDict
from django.utils.datastructures import MultiValueDict

from .response import render_error
from .views import ApiAction

class ApiMiddleware(object):
	"""
	Api middleware. If this is enabled, each request object will
	get the added attributes:
	api (True/False) - depending on the url request if it starts with /api
	api_action (int) - is api is True, ApiAction determined from the HTTP method
	api_json (True/False/String) - depending on the ?json part of the url request
	api_callback (String) - sets request.api_json = callback and request.api_callback = True and is meant render json wrapped in a callback
	api_version (int) - if api is True, 1 by default, > 1 if /api/#/ is given
	NOTE: request.api_callback is only/always set if request.api_json = True and request.api_json is only/always set if request.api = True
	If requested with AJAX or Accept header wants json, then json is always used.
	"""

	_METHOD_ACTION_DICT = {
		'GET': ApiAction.READ,
		'HEAD': ApiAction.READ,
		'POST': ApiAction.CREATE,
		'PUT': ApiAction.UPDATE,
		'PATCH': ApiAction.UPDATE,
		'DELETE': ApiAction.DELETE
	}
	_API_JSONP = getattr(settings, 'API_JSONP', False)
	_API_CSRF = getattr(settings, 'API_CSRF', True)
	_ERROR_JSONP = 'JSONP requests are not allowed'

	def process_request(self, request):
		if request.path.startswith('/api/'):
			request.api = True
			request.api_version = 1
			request.api_action = ApiMiddleware._METHOD_ACTION_DICT.get(request.method, ApiAction._UNKNOWN)
			if request.is_ajax() or request.META.get('HTTP_ACCEPT', '').startswith('application/json'):
				request.api_json = True
			else:
				callback = request.GET.get('callback', None)
				if callback:
					request.api_callback = True
					request.api_json = callback
				else:
					request_json = request.GET.get('json', None)
					if request_json:
						request.api_callback = False
						if request_json.lower() == 'true':
							request.api_json = True
						else:
							request.api_json = request_json
					else:
						request.api_json = False

			# Check for a restricted JSONP request
			if type(request.api_json) is not bool and not ApiMiddleware._API_JSONP:
				return render_error(request, ApiMiddleware._ERROR_JSONP, 403)

			# Detect the requested API version
			components = request.path.split('/', 3)
			if len(components) >= 3 and components[2].isdigit():
				request.api_version = int(components[2])

			# Process JSON data and PUT requests
			if request.META.get('CONTENT_TYPE', '').startswith('application/json'):
				# Should set either request.POST or request.PUT
				query_dict = QueryDict('', mutable=True)
				query_dict.update(json.loads(request.body))
				setattr(request, request.method, query_dict)
				if request.method == 'PATCH':
					request.PUT = request.PATCH
				request._files = MultiValueDict()
			elif request.method == 'PUT':
				# Process PUT requests
				if hasattr(request, '_post'):
					del request._post
					del request._files
				request.method = 'POST'
				request._load_post_and_files()
				request.method = 'PUT'
				request.PUT = request.POST

			# Skip the csrf check (see Django's CsrfViewMiddleware)
			if not ApiMiddleware._API_CSRF and (request.is_ajax() or request.META.has_key('HTTP_X_NATIVE_APP')):
				request.csrf_processing_done = True
		else:
			request.api = False



import os
import re
from importlib import import_module
from optparse import make_option

from django.conf import settings
from django.core.management.base import CommandError
from django.template import Template, Context

import symmetric.management.overrides
from symmetric.functions import _ApiModel
from symmetric.management.functions import format_regex_stack, get_app_url_prefix, is_sublist, get_subclass_filter, get_model_name
from symmetric.views import ApiAction, ApiRequirement, BasicApiView, api_view

"""
	Connection actions are mapped in the following way:
	api_view or ApiView
		with missing URL params - READ_LIST, CREATE
			for a BasicApiView single_object = True or an api_view with default_args for id or slug - READ_OBJECT, UPDATE
		with <object_id> - READ_OBJECT, UPDATE, DELETE
		with <slug> - READ_OBJECT, no update or delete because the object needs to be passed to the method and only the id is used
	api_related_view or ApiRelatedView
		regardless missing or provided URL params if single_object = True for a BasicApiView - READ_RELATED_OBJECT, UPDATE_RELATED
		with missing URL params - not possible for api_related_view
		with <object_id> or <slug> - READ_RELATED_LIST, CREATE_RELATED
	BasicApiViews, including subclasses ApiCurrentUserView, ApiCurrentUserRelatedView
		treated as api_view when parent_model is None
		treated as api_related_view when parent_model is not None

	Current user and collections:
	userinfo - /me - ApiCurrentUserView (BasicApiView single_object=True)
	collection - /me/posts - ApiCurrentUserRelatedView (BasicApiView single_object=False)
	Related single objects can only be done with a BasicApiView e.g.:
	object - /me/currentpost - needs to be implemented by the user as a BasicApiView and parent_model set to user, model set to post and .single_object True

	NOTE: api auth views are not supported

	Generated method signatures should be similar to the following
	READ_OBJECT - readObject
	READ_LIST - readObjects
	CREATE - createObject(object)
	UPDATE - updateObject(object)
	DELETE - deleteObject(object)
	READ_RELATED_LIST - readChildrenForParent
	CREATE_RELATED - createChildForParent(childObject)
	READ_RELATED_OBJECT - readObjectForParent
	UPDATE_RELATED - updateObjectForParent(childObject)

	The ACTIONS needing param info:
	object id/slug: READ_OBJECT, UPDATE
	parent id/slug: READ_RELATED_LIST, CREATE_RELATED, READ_RELATED_OBJECT, UPDATE_RELATED:
		{ param: '' or 'id' or 'slug'}
	UPDATE only needs param info for deciding how to create it's URL, the method signature doesn't change, it always takes an object (not id or slug)
	when param is set add withSlug or withId to the method signatures above and additional id or slug argument
"""

class GenerateConnectionsCommand(object):
	option_list = (
			make_option('--dest',
				type='string',
				dest='dest',
				help='Output all connections for the detected apps with api endpoints and render them into this destination directory.'),
		)
	ALL_ACTIONS = ('READ_OBJECT', 'READ_LIST', 'CREATE', 'UPDATE', 'DELETE', 'READ_RELATED_OBJECT', 'UPDATE_RELATED', 'READ_RELATED_LIST', 'CREATE_RELATED')

	def post_render(self, output):
		return output

	def perform_mapping(self, mapping, format_context):
		if callable(mapping):
			# callable method
			return mapping(format_context)
		elif isinstance(mapping, Template):
			# django template
			return mapping.render(Context(format_context, autoescape=False))
		else:
			# normal python string formatting
			return mapping.format(**format_context)

	def get_view_actions(self, url, view):
		actions = []
		single_object = False
		if hasattr(view, 'parent_model') and view.parent_model:
			# For api_related_view, ApiRelatedView, or BasicApiView
			if isinstance(view, BasicApiView) and view.single_object:
				if view.actions & ApiAction.READ:
					actions.append('READ_RELATED_OBJECT')
				if view.actions & ApiAction.UPDATE:
					actions.append('UPDATE_RELATED')
			else:
				if view.actions & ApiAction.READ:
					actions.append('READ_RELATED_LIST')
				if view.actions & ApiAction.CREATE:
					actions.append('CREATE_RELATED')
		else:
			# For api_view, ApiView, or BasicApiView (without a parent_model)
			if (url.find('<object_id>') == -1 and url.find('<slug>') == -1):
				if (isinstance(view, BasicApiView) and view.single_object) or (url in self.single_object_urls):
					if view.actions & ApiAction.READ:
						actions.append('READ_OBJECT')
					if view.actions & ApiAction.UPDATE:
						actions.append('UPDATE')
				else:
					if view.actions & ApiAction.READ:
						actions.append('READ_LIST')
					if view.actions & ApiAction.CREATE:
						actions.append('CREATE')
			elif url.find('<object_id>') != -1:
				if view.actions & ApiAction.READ:
					actions.append('READ_OBJECT')
				if view.actions & ApiAction.UPDATE:
					actions.append('UPDATE')
				if view.actions & ApiAction.DELETE:
					actions.append('DELETE')
			elif url.find('<slug>') != -1:
				if view.actions & ApiAction.READ:
					actions.append('READ_OBJECT')
		return actions

	def get_param_context(self, url, action):
		context = {'param': '', 'param_lower': '', 'param_type': ''}
		if action in ('READ_OBJECT', 'UPDATE', 'READ_RELATED_OBJECT', 'UPDATE_RELATED', 'READ_RELATED_LIST', 'CREATE_RELATED'):
			if url.find('<object_id>') != -1:
				context['param'] = 'Id'
				context['param_lower'] = 'id'
				context['param_type'] = self.object_id_type
			if url.find('<slug>') != -1:
				context['param'] = 'Slug'
				context['param_lower'] = 'slug'
				context['param_type'] = self.slug_type
		return context

	def get_context(self, app_label):
		context = {'name': app_label, 'name_lower': app_label[0].lower() + app_label[1:] }
		if hasattr(self, 'extra_context'):
			context.update(self.extra_context())

		# Loop over the view mappings
		for mapping_name in self.mappings:
			mapping = self.mappings[mapping_name]
			lines = []
			for url_format, view in self.views.iteritems():
				for action in self.get_view_actions(self.urls[url_format], view):
					line = None
					if mapping.has_key(action):
						format_context = {
							'url': self.urls[url_format],
							'url_format': url_format,
							'view': view,
							'app_name': app_label,
							'app_name_lower': app_label[0].lower() + app_label[1:],
							'model_name': view.model.__name__,
							'name': view.model.__name__,
							'name_lower': view.model.__name__[0].lower() + view.model.__name__[1:],
							'name_plural': view.model._meta.verbose_name_plural.title().replace(' ', ''),
							'name_plural_lower': view.model._meta.verbose_name_plural[0].lower() + view.model._meta.verbose_name_plural.title().replace(' ', '')[1:],
							'login': ((view.requirements & ApiRequirement.LOGIN) and not (action.startswith('READ') and (view.requirements & ApiRequirement.ANONYMOUS_READ))),
							'https': (view.requirements & ApiRequirement.HTTPS),
							'hmac': (view.requirements & ApiRequirement.HMAC)
						}
						subclass_filter = get_subclass_filter(view)
						if subclass_filter:
							format_context['subclasses'] = [(_ApiModel(model).id_field[1], get_model_name(model)) for model in subclass_filter.subclasses]
						format_context.update(self.get_param_context(self.urls[url_format], action))
						if hasattr(view, 'parent_model') and view.parent_model:
							# Related and basic api views
							format_context['parent_name'] = view.parent_model.__name__
							format_context['parent_name_lower'] = view.parent_model.__name__[0].lower() + view.parent_model.__name__[1:]
							format_context['parent_name_plural'] = view.parent_model._meta.verbose_name_plural.title().replace(' ', '')
							format_context['parent_name_plural_lower'] = view.parent_model._meta.verbose_name_plural[0].lower() + view.parent_model._meta.verbose_name_plural.title().replace(' ', '')[1:]
						line = self.perform_mapping(mapping[action], format_context)
					if line is None:
						raise CommandError("No such mapping for %s in %s." % (action, mapping_name))
					elif line:
						line = line.split('\n')
						if not mapping.get('RemoveDuplicates', False) or not is_sublist(lines, line):
							lines.extend(line)
			context[mapping_name] = lines
			if mapping.get('Sort', False):
				context[mapping_name].sort()

		return context

	def make_url_format(self, url_pattern):
		url_pattern = url_pattern.replace('<object_id>', self.object_id_format)
		url_pattern = url_pattern.replace('<slug>', self.slug_format)
		return url_pattern

	def enum_patterns(self, patterns):
		for pattern in patterns:
			if pattern.callback:
				self.regex_stack.append(pattern._regex)
				if isinstance(pattern.callback, (api_view, BasicApiView)):
					url = format_regex_stack(self.regex_stack)
					url_format = self.make_url_format(url)
					self.urls[url_format] = url
					if hasattr(pattern, 'default_args') and (pattern.default_args.has_key('object_id') or pattern.default_args.has_key('slug')):
						self.single_object_urls.add(url)
					self.views[url_format] = pattern.callback
				self.regex_stack.pop()
			else:
				self.regex_stack.append(pattern._regex)
				self.enum_patterns(pattern.url_patterns)
		if self.regex_stack:
			self.regex_stack.pop()

	def render(self, *args, **options):
		if not hasattr(self, 'templates'):
			raise CommandError('No templates set!')
		app_renames = {arg.split('=')[0]:arg.split('=')[1] for arg in args if arg.find('=') != -1}
		if options and options['dest']:
			try:
				os.makedirs(options['dest'])
			except:
				print 'Warning: Overwriting any contents in %s' % options['dest']
			for app_name in settings.INSTALLED_APPS:
				try:
					module = import_module(app_name + '.urls')
					patterns = getattr(module, 'apipatterns', getattr(module, 'urlpatterns', []))
				except:
					continue
				app_label = app_name.split('.')[-1]
				app_rename = app_renames.get(app_label, app_label)
				app_rename = app_rename[0].upper() + app_rename[1:]
				self.views = {}
				self.urls = {}
				self.single_object_urls = set()
				self.regex_stack = [get_app_url_prefix(app_name, patterns)]
				self.enum_patterns(patterns)
				if self.views:
					context = self.get_context(app_rename)
					for i in range(len(self.templates)):
						template = self.templates[i]
						template_extension = self.template_extensions[i]
						path = os.path.join(options['dest'], '%sConnection.%s' % (app_rename, template_extension))
						print 'Rendering %s' % path
						with open(path, 'w') as f:
							f.write(self.post_render(template.render(Context(context, autoescape=False))))
		elif args:
			for app_name in args:
				try:
					module = import_module(app_name + '.urls')
					patterns = getattr(module, 'apipatterns', getattr(module, 'urlpatterns', []))
				except:
					continue
				app_label = app_name.split('.')[-1]
				app_rename = app_renames.get(app_label, app_label)
				app_rename = app_rename[0].upper() + app_rename[1:]
				self.views = {}
				self.urls = {}
				self.single_object_urls = set()
				self.regex_stack = [get_app_url_prefix(app_name, patterns)]
				self.enum_patterns(patterns)
				context = self.get_context(app_rename)
				for template in self.templates:
					print self.post_render(template.render(Context(context, autoescape=False)))
		else:
			raise CommandError("No app or destination directory specified.")



from __future__ import print_function

class CodeEmitter(object):
	def __init__(self, fobj, indent=0):
		if isinstance(fobj, (str, unicode)):
			fobj = open(fobj, 'w')
		self.fobj = fobj
		self.indent = ' ' * indent if indent else '\t'
		self.level = 0

	def __call__(self, *lines):
		for line in lines:
			line = line.strip()
			if not line:
				print('', file=self.fobj)
			else:
				if line[0] == '}' or line[0] == ']':
					self.level = self.level - 1
				if line.startswith('case '):
					print((self.indent * (self.level - 1)) + line, file=self.fobj)
				else:
					print((self.indent * self.level) + line, file=self.fobj)
				if line[-1] == '{' or line[-1] == '[':
					self.level = self.level + 1

	def close(self):
		self.fobj.close()



import json

from django.db.models.query import QuerySet
from django.http import HttpResponse, Http404
from django.shortcuts import render_to_response
from django.template import loader, Context
from django.utils.html import escape

from functions import get_object_data, get_object_list_data
import xml

__NO_CACHE = 'max-age=0, no-cache, no-store, must-revalidate'

def __default_dumps(obj):
	if isinstance(obj, QuerySet):
		# Dumping a QuerySet with this function shouldn't happen, but just in case
		return tuple(obj)
	else:
		data = get_object_data(obj)
		if data:
			return data
		else:
			raise TypeError(repr(obj) + " is not serializable")

def __default_list_dumps(obj):
	if isinstance(obj, QuerySet):
		return tuple(obj)
	else:
		data = get_object_list_data(obj)
		if data:
			return data
		else:
			raise TypeError(repr(obj) + " is not serializable")

def set_response_headers(request, **kwargs):
	if not hasattr(request, 'api_response_headers'):
		request.api_response_headers = kwargs.copy()
	else:
		request.api_response_headers.update(kwargs)

def apply_response_headers(request, response):
	if hasattr(request, 'api_response_headers'):
		for header, value in request.api_response_headers.iteritems():
			response[header] = value
	# Django never automatically adds Content-Length to a response unless ConditionalGetMiddleware is used, so do it here in case the middleware isn't being used
	response['Content-Length'] = str(len(response.content))
	response['Cache-Control'] = __NO_CACHE

def render_data(request, data, status=200):
	"""Render data as xml or json based on the request."""
	if request.api:
		default = __default_list_dumps if isinstance(data, (tuple, list, set, QuerySet)) else __default_dumps
		if not request.api_json:
			response = HttpResponse(content_type='application/xml', status=status)
			xml.dumps(data, response, default=default)
		elif request.api_json is not True:
			response = HttpResponse(content_type='text/javascript', status=status)
			if request.api_callback:
				response.write(request.api_json + '(')
				response.write(json.dumps(data, default=default))
				response.write(');')
			else:
				response.write(request.api_json + ' = ')
				response.write(json.dumps(data, default=default))
				response.write(';')
		else:
			response = HttpResponse(json.dumps(data, default=default), content_type='application/json', status=status)
		apply_response_headers(request, response)
		return response
	else:
		raise Http404

def render_error(request, message, status, code=None):
	"""Render and error message and code."""
	if code is None:
		code = status
	if request.api:
		if not request.api_json:
			response = HttpResponse('<?xml version="1.0" encoding="UTF-8" ?><error><code>%d</code><message>%s</message></error>' % (code, escape(message)), status=status, content_type='application/xml')
		elif request.api_json is not True:
			if request.api_callback:
				response = HttpResponse('%s({"code":%d,"message":%s});' % (request.api_json, code, json.dumps(message)), status=200, content_type='text/javascript')
			else:
				response = HttpResponse('%s={"code":%d,"message":%s};' % (request.api_json, code, json.dumps(message)), status=200, content_type='text/javascript')
		else:
			response = HttpResponse('{"code":%d,"message":%s}' % (code, json.dumps(message)), status=status, content_type='application/json')
		apply_response_headers(request, response)
		return response
	else:
		if code == 404:
			raise Http404
		else:
			return HttpResponse('<html><head><title>Error %d</title></head><body><b>Error %d</b> - %s</body></html>' % (status, code, message), status=status, content_type='text/html')

def render_empty(request):
	"""Render an appropriate empty response."""
	# jQuery will report an error if the empty response isn't given as {} or null
	# see dataType "json" here https://api.jquery.com/jQuery.ajax/
	# 204 No Content requires that nothing be in the body, so cannot be used for a json response, see https://tools.ietf.org/html/rfc2616#section-10.2.5
	if request.api_json:
		response = HttpResponse('null', content_type='application/json')
	else:
		response = HttpResponse(status=204)
	apply_response_headers(request, response)
	return response



from setuptools import setup, find_packages

setup(
	name='django-symmetric',
	version='0.0.2',
	description='RESTful Django API views to ease automation of creating symmetric client-side code.',
	url='https://github.com/symmetricapi/django-symmetric',
	author='mvx24',
	author_email='cram2400@gmail.com',
	license='MIT',
	classifiers=[
		'Development Status :: 3 - Alpha',
		'Environment :: Console',
		'Framework :: Django',
		'License :: OSI Approved :: MIT License',
		'Operating System :: MacOS :: MacOS X',
		'Operating System :: POSIX',
		'Operating System :: Unix',
		'Programming Language :: Python :: 2.7',
		'Programming Language :: Python :: 2 :: Only',
		'Topic :: Database',
		'Topic :: Software Development :: Code Generators',
		'Topic :: Software Development :: Libraries :: Python Modules'
	],
	keywords='django restful api views backbone java android ios',
	packages=find_packages(exclude=['tests']),
	install_requires=['django'],
	package_data={
		'': ['management/templates/*']
	}
)



import hashlib
import hmac

from django.apps import apps
from django.conf import settings
from django.contrib.auth import authenticate, login, logout, get_user_model
from django.db import IntegrityError
from django.db.models.signals import post_delete
from django.db.utils import DEFAULT_DB_ALIAS
from django.http import HttpResponse
from django.utils.http import urlencode
from django.views.decorators.csrf import csrf_exempt

from .filters import filter_as_authorization
from .functions import set_object_data, save_object, _get_api_model
from .response import *

get_model = apps.get_model

class ApiAction:
	_UNKNOWN = 0
	READ = 1
	CREATE = 2
	UPDATE = 4
	DELETE = 8
	ALL = 15

class ApiRequirement:
	LOGIN = 1
	STAFF = 2
	SUPERUSER = 4
	JSONP = 8
	ANONYMOUS_READ = 16
	HMAC = 32
	HTTPS = 64
	NON_USER_REQUIREMENTS = ANONYMOUS_READ | HMAC | HTTPS

__ERROR_BAD_REQUEST = 'Bad request'
__ERROR_NOT_ALLOWED = _ERROR_NOT_ALLOWED = 'Method not allowed'
__ERROR_NOT_AUTHORIZED = 'Not authorized'
__ERROR_NOT_FOUND = 'Not found'
__ERROR_VERIFICATION = 'One or more fields were set with incorrect or unauthorized values'
__ERROR_JSONP = 'JSONP requests on this resource are not allowed'
__ERROR_BAD_CREDENTIALS = 'Incorrect username or password'
__ERROR_INACTIVE_ACCOUNT = 'Account is not active'
__ERROR_AUTH_CHALLENGE = 'Further authentication is required'
__ERROR_HMAC = 'Invalid HMAC'
__ERROR_HTTPS = 'HTTPS is required'
__ERROR_USERNAME_TAKEN = 'Username is already taken'
__ERROR_PASSWORD_MISMATCH = 'Passwords do not match'

__X_HEADER_NEW_OBJECT_ID = 'X-New-Object-Id'
__X_HEADER_USER_ID = 'X-User-Id'

def __exception_error_message(e):
	# NOTE: ValidationError has a message_dict property that could be inspected more deeply but it also has a messages list - just return the first reported error from messages.
	if hasattr(e, 'messages') and e.messages:
		return e.messages[0]
	elif e.args:
		return e.args[0]
	else:
		return 'An unknown error occured'

# HMAC key and salt both as encoded byte strings that hmac requires
__API_HMAC_KEY = getattr(settings, 'API_HMAC_KEY', settings.SECRET_KEY)
if type(__API_HMAC_KEY) is unicode:
	__API_HMAC_KEY = __API_HMAC_KEY.encode('utf-8')
__API_HMAC_SALT = getattr(settings, 'API_HMAC_SALT', '')
if type(__API_HMAC_SALT) is unicode:
	__API_HMAC_SALT = __API_HMAC_SALT.encode('utf-8')

def __check_hmac(request):
	"""Perform the hmac check."""
	if request.META.has_key('HTTP_X_HMAC'):
		# NOTE: request.body is already a raw byte string (that hmac requires) not a unicode object
		nonce = request.META.get('HTTP_X_HMAC_NONCE', '')
		if type(nonce) is unicode:
			nonce = nonce.encode('utf-8')
		message = request.body + __API_HMAC_SALT + nonce
		code = hmac.new(__API_HMAC_KEY, message, hashlib.sha256)
		if code.hexdigest() == request.META['HTTP_X_HMAC']:
			return True
	return False

def _check_requirements(request, requirements):
	if requirements & ApiRequirement.ANONYMOUS_READ and request.api_action == ApiAction.READ:
		# For an anonymous read, ignore any other user requirements and regardless of request.user.is_anonymous()
		requirements = requirements & ApiRequirement.NON_USER_REQUIREMENTS
	if requirements & ApiRequirement.LOGIN:
		if not request.user.is_authenticated():
			return render_error(request, __ERROR_NOT_AUTHORIZED, 401)
		elif not request.user.is_active:
			return render_error(request, __ERROR_INACTIVE_ACCOUNT, 403)
	if requirements & ApiRequirement.STAFF and (not request.user.is_staff or not request.user.is_active):
		return render_error(request, __ERROR_NOT_AUTHORIZED, 403)
	if requirements & ApiRequirement.SUPERUSER and (not request.user.is_superuser or not request.user.is_active):
		return render_error(request, __ERROR_NOT_AUTHORIZED, 403)
	if type(request.api_json) is not bool and not requirements & ApiRequirement.JSONP:
		return render_error(request, __ERROR_JSONP, 403)
	if requirements & ApiRequirement.HMAC and (request.api_action == ApiAction.CREATE or request.api_action == ApiAction.UPDATE) and not __check_hmac(request):
		return render_error(request, __ERROR_HMAC, 403)
	if requirements & ApiRequirement.HTTPS and not request.is_secure() and not settings.DEBUG:
		if request.api_action == ApiAction.READ:
			response = render_error(request, __ERROR_HTTPS, 301)
			response['Location'] = request.build_absolute_uri(request.get_full_path()).replace('http://', 'https://')
		else:
			# Information may have been compromised, so don't redirect
			response = render_error(request, __ERROR_HTTPS, 403)
		return response
	return None

class BasicApiView(object):
	"""Basic api view, that allows custom actions to mapped to instance methods of: read, create, update, and delete."""

	_ACTION_DICT = {
		ApiAction.READ: 'read',
		ApiAction.CREATE: 'create',
		ApiAction.UPDATE: 'update',
		ApiAction.DELETE: 'delete'
	}

	# Set single_object to True if the view only returns on object instead of an collection when there is no object_id or slug argument in the URL - used only for management scripts to inspect
	single_object = False
	requirements = 0

	def __call__(self, request, object_id=None, slug=None):
		# Is the action allowed
		action_method = getattr(self, BasicApiView._ACTION_DICT.get(request.api_action, ''), None)
		if not callable(action_method):
			return render_error(request, _ERROR_NOT_ALLOWED, 405)

		# Does the user pass the requirements
		if self.requirements:
			response = _check_requirements(request, self.requirements)
			if response:
				return response

		return action_method(request, object_id, slug)

	@property
	def actions(self):
		"""Actions for management scripts to inspect."""
		actions = 0
		for action, method in BasicApiView._ACTION_DICT.iteritems():
			if hasattr(self, method):
				actions |= action
		return actions

	@property
	def filter(self):
		"""Override and return a filter for management scripts to inspect."""
		return None

	@property
	def authorization(self):
		"""Override and return an authorization for management scripts to inspect."""
		return None

	@property
	def verification(self):
		"""Override and return a verification for management scripts to inspect."""
		return None

	@property
	def parent_model(self):
		"""Override and return a the parent model for management scripts to inspect and generate client connection code treated similar to an api_related_view."""
		return None

	@property
	def model(self):
		"""Override and return a model for management scripts to inspect."""
		return None

def api_view(model, actions=ApiAction.READ, requirements=0, filter=None, authorization=None, verification=None):
	"""Generate an api_view with certain requirements and options."""
	if isinstance(model, (str, unicode)):
		model = model.split('.')
		model = get_model(model[0], model[1])
	slug_field = 'slug'
	deleted_field = None
	nonce_field = None
	request_user_field = None
	request_ip_field = None
	if not authorization and filter:
		authorization = filter_as_authorization(model, filter)
	if hasattr(model, 'API'):
		if hasattr(model.API, 'slug_field'):
			slug_field = model.API.slug_field
		if hasattr(model.API, 'deleted_field'):
			deleted_field = model.API.deleted_field
		if hasattr(model.API, 'nonce_field') and requirements & ApiRequirement.HMAC:
			nonce_field = model.API.nonce_field
		if hasattr(model.API, 'request_user_field'):
			request_user_field = model.API.request_user_field
		if hasattr(model.API, 'request_ip_field'):
			request_ip_field = model.API.request_ip_field

	def api_view_inner(request, object_id=None, slug=None):
		"""The api view that automatically processes RESTful requests."""
		if object_id:
			object_id = int(object_id)

		# Is the action allowed
		if not request.api_action & actions:
			return render_error(request, __ERROR_NOT_ALLOWED, 405)

		# Does the user pass the requirements
		if requirements:
			response = _check_requirements(request, requirements)
			if response:
				return response

		if not request.api:
			return render_error(request, __ERROR_BAD_REQUEST, 400)
		elif request.api_action == ApiAction.READ:
			# Get an existing object or collection
			if object_id or slug:
				try:
					select_related_args = _get_api_model(model).select_related_args
					if select_related_args:
						if object_id:
							obj = model.objects.select_related(*select_related_args).get(id=object_id)
						else:
							obj = model.objects.select_related(*select_related_args).get(**{slug_field: slug})
					else:
						if object_id:
							obj = model.objects.get(id=object_id)
						else:
							obj = model.objects.get(**{slug_field: slug})
					if deleted_field and getattr(obj, deleted_field):
						return render_error(request, __ERROR_NOT_FOUND, 404)
					if callable(authorization) and not authorization(request, obj):
						return render_error(request, __ERROR_NOT_AUTHORIZED, 403)
				except:
					return render_error(request, __ERROR_NOT_FOUND, 404)
				else:
					return render_data(request, obj)
			else:
				# Get a collection
				select_related_args = _get_api_model(model).select_related_args
				if select_related_args:
					if callable(filter):
						queryset = filter(request, model.objects.select_related(*select_related_args))
					else:
						queryset = model.objects.select_related(*select_related_args).all()
				else:
					if callable(filter):
						queryset = filter(request, model.objects.all())
					else:
						queryset = model.objects.all()
				if deleted_field:
					queryset = queryset.filter(**{deleted_field: False})
				return render_data(request, queryset)
		elif request.api_action == ApiAction.CREATE:
			# Create a new object on a collection only
			if object_id or slug:
				return render_error(request, __ERROR_NOT_ALLOWED, 405)
			else:
				try:
					obj = model()
					set_object_data(obj, request.POST)
					if nonce_field:
						nonce = request.META.get('HTTP_X_HMAC_NONCE', None)
						if nonce:
							setattr(obj, nonce_field, nonce)
						else:
							return render_error(request, __ERROR_HMAC, 403)
					if request_user_field and not request.user.is_anonymous():
						setattr(obj, request_user_field, request.user)
					if request_ip_field:
						setattr(obj, request_ip_field, request.META['REMOTE_ADDR'])
					if request.POST.has_key('_data'):
						obj._data = request.POST['_data']
					if callable(verification) and not verification(request, obj):
						return render_error(request, __ERROR_VERIFICATION, 500)
					save_object(obj)
				except Exception as e:
					return render_error(request, '%s: %s' % (e.__class__.__name__, __exception_error_message(e)), 500)
				set_response_headers(request, **{__X_HEADER_NEW_OBJECT_ID: obj.id})
				return render_data(request, {_get_api_model(model).id_field[1]: obj.id}, 201)
		elif request.api_action == ApiAction.UPDATE:
			# Update an existing object only
			if object_id or slug:
				try:
					select_related_args = _get_api_model(model).select_related_args
					if select_related_args:
						if object_id:
							obj = model.objects.select_related(*select_related_args).get(id=object_id)
						else:
							obj = model.objects.select_related(*select_related_args).get(**{slug_field: slug})
					else:
						if object_id:
							obj = model.objects.get(id=object_id)
						else:
							obj = model.objects.get(**{slug_field: slug})
					if callable(authorization) and not authorization(request, obj):
						return render_error(request, __ERROR_NOT_AUTHORIZED, 403)
				except:
					return render_error(request, __ERROR_NOT_FOUND, 404)
				else:
					try:
						set_object_data(obj, request.PUT)
						if request_user_field and not request.user.is_anonymous():
							setattr(obj, request_user_field, request.user)
						if request_ip_field:
							setattr(obj, request_ip_field, request.META['REMOTE_ADDR'])
						if request.PUT.has_key('_data'):
							obj._data = request.PUT['_data']
						if callable(verification) and not verification(request, obj):
							return render_error(request, __ERROR_VERIFICATION, 500)
						save_object(obj)
					except Exception as e:
						return render_error(request, '%s: %s' % (e.__class__.__name__, __exception_error_message(e)), 500)
					return render_empty(request)
			else:
				return render_error(request, __ERROR_NOT_ALLOWED, 405)
		elif request.api_action == ApiAction.DELETE:
			# Delete an existing object only
			if object_id or slug:
				try:
					if object_id:
						obj = model.objects.get(id=object_id)
					else:
						obj = model.objects.get(**{slug_field: slug})
					if callable(authorization) and not authorization(request, obj):
						return render_error(request, __ERROR_NOT_AUTHORIZED, 403)
				except:
					return render_error(request, __ERROR_NOT_FOUND, 404)
				else:
					if deleted_field:
						# Set as deleted only if it is not already
						if not getattr(obj, deleted_field):
							setattr(obj, deleted_field, True)
							obj.save(update_fields=(deleted_field,))
							# Send the post_delete signal
							post_delete.send_robust(sender=obj.__class__, instance=obj, using=DEFAULT_DB_ALIAS)
					else:
						obj.delete()
					return render_empty(request)
			else:
				return render_error(request, __ERROR_NOT_ALLOWED, 405)

	return api_view_inner

def api_related_view(model, related_model, related_field, actions=ApiAction.READ, requirements=0, filter=None, authorization=None, verification=None):
	"""Returns a view got getting a collection of related elements, or POSTing a new one. Other operations are not allowed."""

	# Do not allow UPDATE or DELETE
	if actions & ApiAction.UPDATE:
		actions -= ApiAction.UPDATE
	if actions & ApiAction.DELETE:
		actions -= ApiAction.DELETE

	if isinstance(model, (str, unicode)):
		model = model.split('.')
		model = get_model(model[0], model[1])
	if isinstance(related_model, (str, unicode)):
		related_model = related_model.split('.')
		related_model = get_model(related_model[0], related_model[1])
	slug_field = 'slug'
	deleted_field = None
	if hasattr(model, 'API'):
		if hasattr(model.API, 'slug_field'):
			slug_field = model.API.slug_field
		if hasattr(model.API, 'deleted_field'):
			deleted_field = model.API.deleted_field

	nonce_field = None
	request_user_field = None
	request_ip_field = None
	if hasattr(related_model, 'API'):
		if hasattr(related_model.API, 'nonce_field') and requirements & ApiRequirement.HMAC:
			nonce_field = related_model.API.nonce_field
		if hasattr(related_model.API, 'request_user_field'):
			request_user_field = related_model.API.request_user_field
		if hasattr(related_model.API, 'request_ip_field'):
			request_ip_field = related_model.API.request_ip_field

	def api_related_view_filter(request, queryset):
		if request.api_related_id:
			queryset = queryset.filter(**{related_field + '_id': request.api_related_id})
		else:
			queryset = queryset.filter(**{related_field + '__' + slug_field: request.api_related_slug})
		if callable(filter):
			queryset = filter(request, queryset)
		return queryset

	related_view = api_view(related_model, actions, requirements=0, filter=api_related_view_filter)

	def api_related_view_inner(request, object_id=None, slug=None):
		if object_id:
			object_id = int(object_id)

		# Is the action allowed, will be repeated in related_view, but that is ok
		if not request.api_action & actions:
			return render_error(request, __ERROR_NOT_ALLOWED, 405)

		# Does the user pass the requirements, won't be repeated in related_view, because requirements=0
		if requirements:
			response = _check_requirements(request, requirements)
			if response:
				return response

		if not request.api:
			return render_error(request, __ERROR_BAD_REQUEST, 400)
		else:
			# Check for object existence and authorization first
			try:
				if object_id:
					obj = model.objects.get(id=object_id)
				else:
					obj = model.objects.get(**{slug_field: slug})
				if deleted_field and getattr(obj, deleted_field):
					return render_error(request, __ERROR_NOT_FOUND, 404)
				if callable(authorization) and not authorization(request, obj):
					return render_error(request, __ERROR_NOT_AUTHORIZED, 403)
			except:
				return render_error(request, __ERROR_NOT_FOUND, 404)
			if request.api_action == ApiAction.CREATE:
				try:
					# Create a new related object
					related_obj = related_model()
					set_object_data(related_obj, request.POST)
					setattr(related_obj, related_field, obj)
					if nonce_field:
						nonce = request.META.get('HTTP_X_HMAC_NONCE', None)
						if nonce:
							setattr(related_obj, nonce_field, nonce)
						else:
							return render_error(request, __ERROR_HMAC, 403)
					if request_user_field and not request.user.is_anonymous():
						setattr(related_obj, request_user_field, request.user)
					if request_ip_field:
						setattr(related_obj, request_ip_field, request.META['REMOTE_ADDR'])
					if request.POST.has_key('_data'):
						related_obj._data = request.POST['_data']
					if callable(verification) and not verification(request, related_obj):
						return render_error(request, __ERROR_VERIFICATION, 500)
					save_object(related_obj)
				except Exception as e:
					return render_error(request, '%s: %s' % (e.__class__.__name__, __exception_error_message(e)), 500)
				set_response_headers(request, **{__X_HEADER_NEW_OBJECT_ID: related_obj.id})
				return render_data(request, {_get_api_model(related_model).id_field[1]: related_obj.id}, 201)
			else:
				# Do not pass on the model object_id or slug to read the related objects, but save them in the request for access in the filters
				request.api_related_id = object_id
				request.api_related_slug = slug
				return related_view(request)

	return api_related_view_inner

class ApiView(object):
	actions = ApiAction.READ
	requirements = 0

	@classmethod
	def as_view(cls, **initkwargs):
		instance = cls(**initkwargs)
		if hasattr(instance, 'filter') and callable(instance.filter):
			filter = instance.filter
		else:
			filter = None
		if hasattr(instance, 'authorization') and callable(instance.authorization):
			authorization = instance.authorization
		else:
			authorization = None
		if hasattr(instance, 'verification') and callable(instance.verification):
			verification = instance.verification
		else:
			verification = None
		return api_view(instance.model, instance.actions, instance.requirements, filter, authorization, verification)

class ApiRelatedView(object):
	actions = ApiAction.READ
	requirements = 0

	@classmethod
	def as_view(cls, **initkwargs):
		instance = cls(**initkwargs)
		if hasattr(instance, 'filter') and callable(instance.filter):
			filter = instance.filter
		else:
			filter = None
		if hasattr(instance, 'authorization') and callable(instance.authorization):
			authorization = instance.authorization
		else:
			authorization = None
		if hasattr(instance, 'verification') and callable(instance.verification):
			verification = instance.verification
		else:
			verification = None
		return api_related_view(instance.model, instance.related_model, instance.related_field, instance.actions, instance.requirements, filter, authorization, verification)

class AuthChallenge(Exception):
	"""The authentication request needs more info."""
	def __init__(self, **kwargs):
		self.args = kwargs
	def __str__(self):
		return repr(self.args)

@csrf_exempt
def api_login_view(request):
	"""Login a user based on some credentials.

	POST with HTTPS: args that correspond to credentials passed to the authentication backends.
	"""
	if request.method != 'POST':
		return render_error(request, __ERROR_NOT_ALLOWED, 405)
	if not request.is_secure() and not settings.DEBUG:
		return render_error(request, __ERROR_HTTPS, 403)
	try:
		user = authenticate(**request.POST.dict())
	except AuthChallenge as challenge:
		set_response_headers(request, **challenge.args)
		return render_error(request, __ERROR_AUTH_CHALLENGE, 401)
	else:
		if user is None:
			return render_error(request, __ERROR_BAD_CREDENTIALS, 403)
		elif not user.is_active:
			return render_error(request, _ERROR_INACTIVE_ACCOUNT, 403)
		else:
			login(request, user)
			set_response_headers(request, **{__X_HEADER_USER_ID: user.id})
			# Tell the CsrfViewMiddleware to add the csrftoken cookie to the response (see Django's @ensure_csrf_cookie and get_token())
			if getattr(settings, 'API_CSRF', True):
				request.META['CSRF_COOKIE_USED'] = True
			return render_empty(request)

def api_logout_view(request):
	"""Logout the current user."""
	logout(request)
	return render_empty(request)

class ApiCurrentUserView(BasicApiView):
	single_object = True

	def __init__(self, actions=ApiAction.READ|ApiAction.UPDATE, requirements=ApiRequirement.LOGIN):
		self._actions = actions
		self.requirements = requirements
		if hasattr(self, 'verification') and callable(self.verification):
			verification = self.verification
		else:
			verification = None
		self.api_view = api_view(get_user_model(), self._actions, self.requirements, verification=verification)

	def __call__(self, request):
		"""Return the current user object."""
		return self.api_view(request, request.user.id)

	@property
	def actions(self):
		return self._actions

	@property
	def model(self):
		return get_user_model()

class ApiCurrentUserRelatedView(BasicApiView):
	single_object = False

	def __init__(self, related_model, related_field, actions=ApiAction.READ|ApiAction.CREATE, requirements=ApiRequirement.LOGIN):
		if isinstance(related_model, (str, unicode)):
			related_model = related_model.split('.')
			related_model = get_model(related_model[0], related_model[1])
		self.related_model = related_model
		self.related_field = related_field
		self._actions = actions
		self.requirements = requirements
		if hasattr(self, 'filter') and callable(self.filter):
			filter = self.filter
		else:
			filter = None
		if hasattr(self, 'authorization') and callable(self.authorization):
			authorization = self.authorization
		else:
			authorization = None
		if hasattr(self, 'verification') and callable(self.verification):
			verfication = self.verification
		else:
			verification = None
		self.api_related_view = api_related_view(get_user_model(), self.related_model, self.related_field, self._actions, self.requirements, filter, authorization, verification)

	def __call__(self, request):
		"""Only return objects associated with the current user."""
		return self.api_related_view(request, request.user.id)

	@property
	def actions(self):
		return self._actions

	@property
	def parent_model(self):
		return get_user_model()

	@property
	def model(self):
		return self.related_model

def api_filter_contacts_view(requirements=ApiRequirement.LOGIN|ApiRequirement.HTTPS, fields=('email',)):
	"""Filter a list of contacts based on an arbitrary user model fields such as email.

	POST with LOGIN and HTTPS: args are arrays of values keyed with the field names. The response should be the same just filtered.
	"""
	def api_filter_contacts_view_inner(request):
		if request.method != 'POST':
			return render_error(request, __ERROR_NOT_ALLOWED, 405)
		response = _check_requirements(request, requirements)
		if response:
			return response
		filtered = {}
		UserModel = get_user_model()
		for field in fields:
			values = request.POST.get(field)
			if values:
				filtered[field] = UserModel.objects.filter(**{field + '__in': values}).values_list(field, flat=True)
		if request.META.get('HTTP_ACCEPT', '').startswith('text/csv'):
			values = filtered.values()[0]
			if len(values):
				return HttpResponse(','.join([str(v) for v in values]), content_type='text/csv')
			else:
				return HttpResponse()
		else:
			return render_data(request, filtered)
	return api_filter_contacts_view_inner

@csrf_exempt
def api_create_user_view(request):
	"""Create a new user with a password and optional email. If successful will login that user and return their new user id.

	POST with HTTPS and HMAC: username, password1, password2, email
	"""
	if request.method != 'POST':
		return render_error(request, __ERROR_NOT_ALLOWED, 405)
	response = _check_requirements(request, ApiRequirement.HTTPS | ApiRequirement.HMAC)
	if response:
		return response
	username = request.POST.get('username')
	email = request.POST.get('email')
	password1 = request.POST.get('password1')
	password2 = request.POST.get('password2')
	UserModel = get_user_model()
	user = UserModel(**{UserModel.USERNAME_FIELD: username})
	if email:
		user.email = email
	user.clean()
	try:
		existing_user = UserModel._default_manager.get_by_natural_key(getattr(user, UserModel.USERNAME_FIELD))
		return render_error(request, __ERROR_USERNAME_TAKEN, 409)
	except UserModel.DoesNotExist:
		try:
			if not password1 or password1 != password2:
				return render_error(request, __ERROR_PASSWORD_MISMATCH, 400)
			user.set_password(password1)
			user.save()
		except IntegrityError:
			# Improbable race condition, where the username is now taken
			return render_error(request, __ERROR_USERNAME_TAKEN, 409)
		except Exception as e:
			return render_error(request, '%s: %s' % (e.__class__.__name__, __exception_error_message(e)), 500)
		else:
			user.backend = 'django.contrib.auth.backends.ModelBackend'
			login(request, user)
			set_response_headers(request, **{__X_HEADER_USER_ID: user.id})
			# Tell the CsrfViewMiddleware to add the csrftoken cookie to the response
			if getattr(settings, 'API_CSRF', True):
				request.META['CSRF_COOKIE_USED'] = True
			return render_empty(request)

def api_set_password_view(request):
	"""Change the current user's password.

	POST with LOGIN and HTTPS: password, password1, password2
	"""
	if request.method != 'POST':
		return render_error(request, __ERROR_NOT_ALLOWED, 405)
	response = _check_requirements(request, ApiRequirement.LOGIN | ApiRequirement.HTTPS)
	if response:
		return response
	user = request.user
	old_password = request.POST.get('password')
	password1 = request.POST.get('password1')
	password2 = request.POST.get('password2')
	if user.has_usable_password() and not user.check_password(old_password):
		return render_error(request, __ERROR_BAD_CREDENTIALS, 400)
	elif not password1 or password1 != password2:
		return render_error(request, __ERROR_PASSWORD_MISMATCH, 400)
	user.set_password(password1)
	user.save()
	return render_empty(request)

@csrf_exempt
def api_reset_password_view(request):
	"""Send in a password reset request, where a link will be emailed to the email specified in the POST and recipient will need to set their password in a browser.

	POST with HMAC: email
	"""
	if request.method != 'POST':
		return render_error(request, __ERROR_NOT_ALLOWED, 405)
	response = _check_requirements(request, ApiRequirement.HMAC)
	if response:
		return response
	from django.contrib.auth.forms import PasswordResetForm
	form = PasswordResetForm(request.POST)
	form.save()
	return render_empty(request)

def api_require_login_js(request):
	"""Return a piece of javascript to redirect a user first thing if they are not logged in when loading up a statically hosted SPA."""
	if request.user.is_authenticated() and request.user.is_active:
		response = HttpResponse('void(0);', content_type='text/javascript')
	else:
		login_href = "%s?%s" % (settings.LOGIN_URL, urlencode({'next': settings.LOGIN_REDIRECT_URL}))
		response = HttpResponse('window.location.href = "%s";' % login_href, content_type='text/javascript')
	return response
